@@ -1,4 +1,4 @@
-#' Import data from a Loligo Systems Witrox transmitter
+#' Import data from a Loligo Systems Witrox O2 transmitter
 #'
 #' Imports the standard txt file output from Loligo Systems Witrox fiber optic O2 transmitters and converts the data into one or more data frames.
 #'
@@ -9,7 +9,7 @@
 #' If you would like support for the Witrox 1, email me a data file from this device.
 #'
 #' @param file a character string. The filepath for the file to be read.
-#' @param o2_unit a character string. The unit of O2 measurement to be output in the data.frame. Options are described in \code{\link{conv_o2}}
+#' @param o2_unit a character string. The unit of O2 measurement to be output in the data frame. Options are described in \code{\link{conv_o2}}.
 #' @param date a character string. The date format to be passed to \code{\link{strptime}}.
 #' @param overwrite_sal Default \code{NULL}. To overwrite the salinity value(s) from calibration, enter a single numeric value for all channels or a numeric vector with values for each channel. Salinity of water sample (psu).
 #' @param drop_channels logical. Should channels without any O2 data be dropped? Default is \code{TRUE}.
@@ -28,8 +28,7 @@
 #' }
 #' If \code{split_channels = TRUE}, then "\code{CH_X_}" is removed from the column names and multiple data frames are returned in a list.
 #' @author Matthew A. Birk, \email{matthewabirk@@gmail.com}
-#' @note Oxygen conversions are estimates based on the \code{\link[marelac]{marelac}} package.
-#' @seealso \code{\link{import_presens}}, \code{\link{conv_o2}}
+#' @seealso \code{\link{import_firesting}}, \code{\link{import_presens}}, \code{\link{conv_o2}}
 #' @examples
 #' \dontrun{
 #' file <- system.file('extdata', 'witrox_file.txt', package = 'respirometry')
@@ -40,7 +39,7 @@
 #' import_witrox(file, o2_unit = 'umol_per_kg', overwrite_sal = 35)
 #' 
 #' # I want each channel as a separate data frame.
-#' data_list = import_witrox(file, split_channels = TRUE)
+#' data_list <- import_witrox(file, split_channels = TRUE)
 #' data_list$CH_3 # here's the channel 3 data frame.
 #' }
 #'
@@ -48,10 +47,11 @@
 #' @export
 
 import_witrox = function(file, o2_unit = 'percent_a.s.', date = '%m/%d/%Y %I:%M:%S %p', overwrite_sal = NULL, drop_channels = TRUE, split_channels = FALSE){
-	raw = readLines(file)
+	raw = readChar(file, nchars = file.info(file)$size, useBytes = TRUE)
 	raw = gsub(pattern = '\xb0|\xa9|\xfc\xbe\x8e\x93\xa0\xbc', replacement = ' ', raw) # replace non ASCII characters
+	raw = strsplit(raw, split = '\r\n', fixed = T)[[1]]
 	raw = raw[sapply(raw, nchar) > 0] # remove blank rows
-	f = strsplit(raw, split = '\t')
+	f = strsplit(raw, split = '\t', fixed = TRUE)
 	sals = as.numeric(sapply(f[grep('CALIBRATION DATA', f[1:20]) + 2:5], '[', 9))
 	if(is.null(overwrite_sal)) overwrite_sal = sals
 	if(length(overwrite_sal) == 1) overwrite_sal = rep(overwrite_sal, length(sals))
@@ -89,21 +89,23 @@ import_witrox = function(file, o2_unit = 'percent_a.s.', date = '%m/%d/%Y %I:%M:
 	o2_cols = grep('CH_\\d_O2', colnames(f)) # update now that SAL exists
 	if(any(is.na(f$TIME))) stop(paste('The time record does not match', date, 'on at least some of the lines between', range(which(is.na(f$TIME)))[1], 'and', range(which(is.na(f$TIME)))[2]), call. = FALSE)
 	row.names(f) = NULL
-	if(drop_channels == TRUE){
+	if(drop_channels){
 		blank_cols = apply(f[, o2_cols], 2, function(i) all(is.na(i)))
 		if(any(blank_cols)){
-			drop = gsub('_O2', '', names(which(blank_cols)))
-			f = f[, rowSums(sapply(drop, function(i) grepl(i, colnames(f)))) == 0]
+			drop = gsub('_O2$', '', names(which(blank_cols)))
+			f = f[, !grepl(paste(drop, collapse = '|'), colnames(f))]
 		}
 	}
-	if(split_channels == TRUE){
+	if(split_channels){
 		channel_names = unique(gsub('(^CH_\\d).*', '\\1', grep('^CH_\\d', colnames(f), value = TRUE)))
 		f = lapply(0:(sum(grepl('^CH_', colnames(f))) / 4 - 1), function(i){
 			f_sub = f[, c(grep('^CH_', colnames(f), value = TRUE, invert = TRUE), grep('^CH_', colnames(f), value = TRUE)[1:4 + 4 * i])]
 			colnames(f_sub) = gsub('^CH_\\d_(.*)', '\\1', colnames(f_sub))
-			if(any(!is.na(f_sub$O2))) f_sub = f_sub[birk::range_seq(which(is.na(f_sub$O2) == FALSE)), ]
-			f_sub$DURATION = as.numeric(difftime(time1 = f_sub$TIME, time2 = f_sub[1, 'TIME'], units = 'mins'))
-			row.names(f_sub) = NULL
+			if(any(is.na(f_sub$O2))){
+				f_sub = f_sub[birk::range_seq(which(is.na(f_sub$O2) == FALSE)), ]
+				f_sub$DURATION = as.numeric(difftime(time1 = f_sub$TIME, time2 = f_sub[1, 'TIME'], units = 'mins'))
+				row.names(f_sub) = NULL
+			}
 			return(f_sub)
 		})
 		names(f) = channel_names
